"""
DB 와 연결하는 부분
"""

import pandas as pd

from sqlalchemy.orm.session import sessionmaker

from logging import getLogger

from pycontainerutils.db.BaseDB_Adapter import BaseDBAdapter

logger = getLogger(__name__)


class DBAdapter(BaseDBAdapter):
    """
    데이터를 사용하는 엔진
    해당 데이터를 다루는 모든 조작을 담당
    """

    def __init__(self, name, db_name=None, direct=None):
        super().__init__()
        # 이름 설정
        self.name = name
        self.register_info(db_name=db_name, direct=direct)

    def execute_sql(self, sql: str):
        """
        sql문 단순 실행
        :param sql:
        :return:
        """
        print(f"{self.name} : {sql}")
        session = self.session_maker()
        try:
            session.execute(sql)
            session.commit()
        except Exception as e:
            session.rollback()
            logger.error(f"execute_sql fail \n"
                         f"{self.name} engine {self.db_name} - {self.engine_info}")
            raise e
        finally:
            session.close()

    def fetch_data_by_sql(self, sql: str):
        """
        sql문 단순 실행
        :param sql:
        :return:
        """
        print(f"{self.name} : {sql}")
        session = self.session_maker()
        try:
            result_proxy = session.execute(sql)
            return result_proxy.fetchall()
        except Exception as e:
            session.rollback()
            logger.error(f"execute_sql fail \n"
                         f"{self.name} engine {self.db_name} - {self.engine_info}")
            raise e
        finally:
            session.close()

    def fetch_df_by_sql(self, sql: str):
        """
        sql문 단순 실행
        :param sql:
        :return:
        """
        print(f"{self.name} : {sql}")
        session = self.session_maker()
        try:
            data = pd.read_sql(sql, session.bind)
            return data
        except Exception as e:
            session.rollback()
            logger.error(f"execute_sql fail\n"
                         f"{self.name} engine {self.db_name} - {self.engine_info}")
            raise e
        finally:
            session.close()

    def insert_df(self, table_name: str, data: pd.DataFrame, **kwargs):
        data.to_sql(
            table_name,
            self.engine,
            index=False,
            if_exists='append',
            **kwargs
        )



from logging import getLogger
from sqlalchemy.engine.create import create_engine
from sqlalchemy.orm.session import sessionmaker

logger = getLogger(__name__)


class BaseDBAdapter(object):
    name = "BaseDBAdapter"
    # 연결된 DB 정보
    default_db_name = None
    # 세부 설정 사항
    is_echo_sql = False
    databases = None

    def __init__(self):
        # db 연결 정보
        self.engine_info = None
        self.db_name = None
        # db 연결 객체
        self.engine = None
        self.session_maker = None

    @classmethod
    def init_settings(cls, databases, is_echo_sql, default_db_name):
        """
        초기 설정
        :param databases: databases 목록
        :param is_echo_sql: sql echo 여부
        :param default_db_name: 기본으로 사용할 db 이름
        :return:
        """
        cls.databases = databases
        cls.is_echo_sql = is_echo_sql
        if default_db_name in databases:
            cls.default_db_name = default_db_name
        else:
            print(f"잘못된 {default_db_name}(default_db_name) 입니다.")
            raise SyntaxError

    def create_db_connect(self):
        db_link = f'postgresql://{self.engine_info["user"]}:{self.engine_info["password"]}' \
                  f'@{self.engine_info["host"]}:{self.engine_info["port"]}/{self.engine_info["database"]}'
        print(f"{self.name} engine create: {self.db_name} - {self.engine_info}")

        # engine 생성
        self.engine = create_engine(db_link, echo=self.is_echo_sql)
        # DB 연결 create
        self.session_maker = sessionmaker(bind=self.engine)

    def register_info(self, db_name, direct):
        # 둘다 입력한 경우
        if db_name and direct:
            logger.error(f"db 이름과 direct를 통해 이중으로 연결되었습니다.\n"
                         f"db_name: {db_name}"
                         f"direct: {direct}")
            raise SyntaxError
        # db name을 입력한 경우
        if db_name is not None:
            self.register_info_db_name(db_name=db_name)
        # 직접 ip를 입력한 경우
        if direct is not None:
            self.register_info_direct(direct=direct)
        # 아무것도 입력하지 않은 경우
        if not db_name and not direct:
            self.register_info_db_name(db_name=self.default_db_name)

        # 연결 생성
        self.create_db_connect()

    def register_info_db_name(self, db_name):
        if db_name in self.databases:
            self.engine_info = self.databases[db_name]
            self.db_name = db_name
        else:
            # 기본 입력된 정도 등록
            # database에서 조회해서 등록
            logger.warning(f"해당 DB 이름을 settings의 DATABASES에서 찾을 수 없습니다.")
            raise KeyError

    def register_info_direct(self, direct):
        # 직접 입력
        self.engine_info = {
            "database": direct['database'],
            "user": direct['user'],
            "password": direct['password'],
            "host": direct['host'],
            "port": direct['port'],
        }
        self.db_name = 'direct'
        logger.warn(f"직접 db 정보를 입력하셨습니다.\n"
                    f"{self.engine_info}")




base_logger_config = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "simple": {
            '()': 'coloredlogs.ColoredFormatter',  # colored output

            "format": "[%(asctime)s] %(levelname)s :%(name)s:%(lineno)s %(message)s",
            "datefmt": "%Y-%m-%d %H:%M:%S"
        },
        "complex": {
            '()': 'coloredlogs.ColoredFormatter',  # colored output
            "format": "[%(asctime)s] %(levelname)s [%(module)s %(name)s :%(lineno)s] %(message)s",
            "datefmt": "%Y-%m-%d %H:%M:%S"
        },
    },
    "handlers": {
        "console": {
            "class": "logging.StreamHandler",
            "formatter": "complex",
            "level": "DEBUG",
            "stream": "ext://sys.stdout",
        },
        "db": {
            "class": "pycontainerutils.logger.db.DB_handler.DatabaseHandler",
            "level": "NOTSET",
        },
        # "errors_file": {
        #     "class": "logging.FileHandler",
        #     "filename": "logs/error.log",
        #     "formatter": "complex",
        #     "level": "WARNING",
        # },
        # "all_file": {
        #     "class": "logging.handlers.TimedRotatingFileHandler",
        #     "level": "DEBUG",
        #     "formatter": "complex",
        #     "filename": "logs/all/all.log",
        #     "when": "midnight",
        #     "backupCount": 30,
        #     "interval": 1,
        #     "encoding": "utf-8"
        # },
    },
}


import setuptools

import unittest


def my_test_suite():
    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover('tests', pattern='test_*.py')
    return test_suite


setuptools.setup(
    name="pycontainerutils",  # 패키지 명
    version='0.0.0.4',
    license='MIT',
    author='greenrain',
    author_email='kdwkdw0078@gmail.com',
    description='docker container utilities',
    long_description=open('README.md', encoding='UTF-8').read(),
    url="https://github.com/greenrain78",
    packages=setuptools.find_packages(),
    test_suite='setup.my_test_suite',
    install_requires=[
        "coloredlogs==15.0.1",
        "greenlet==1.1.2",
        "humanfriendly==10.0",
        "numpy==1.21.4",
        "pandas==1.3.4",
        "psycopg2==2.9.2",
        "pyreadline3==3.3",
        "python-dateutil==2.8.2",
        "pytz==2021.3",
        "six==1.16.0",
        "SQLAlchemy==1.4.27",
    ]
)



import traceback
from datetime import datetime
from logging import getLogger, Handler

from sqlalchemy.engine.create import create_engine
from sqlalchemy.orm.session import Session, sessionmaker

logger = getLogger(__name__)


class DatabaseHandler(Handler):
    engine = None
    db_table = f"app_model_logs"
    container = None

    session_maker = None

    def __init__(self, level=0):
        super().__init__(level)

    @classmethod
    def init_db_adapter(cls, container, db_info):
        cls.container = container
        # logger 는 별로의 코드를 사용하여 DB 연결
        db_link = f'postgresql://{db_info["user"]}:{db_info["password"]}' \
                  f'@{db_info["host"]}:{db_info["port"]}/{db_info["database"]}'
        cls.engine = create_engine(db_link)
        cls.session_maker = sessionmaker(bind=cls.engine)

        create_table_sql = f"CREATE TABLE IF NOT EXISTS {cls.db_table}(" \
                           "id           serial  primary key      not null," \
                           "container    varchar(50)              not null," \
                           "process      integer                  not null," \
                           "process_name varchar(50)              not null," \
                           "thread_name  varchar(50)              not null," \
                           "name         varchar(50)              not null," \
                           "thread       bigint                   not null," \
                           "pathname     varchar(200)             not null," \
                           "func_name    varchar(50)              not null," \
                           "lineno       integer                  not null," \
                           "levelname    varchar(50)              not null," \
                           "time         timestamp with time zone not null," \
                           "message      text                     not null," \
                           "exc_info     text                     not null," \
                           "exc_text     text                     not null," \
                           "stack_info   text                     not null," \
                           "levelno      integer                  not null" \
                           ");"
        cls.execute_sql(create_table_sql)

    def __del__(self):
        # db 연결 제거
        try:
            if self.container is not None:
                self.engine.dispose()
        except Exception:
            traceback.print_exc()
        else:
            self.engine = None

    @classmethod
    def execute_sql(cls, sql: str):
        """
        sql문 단순 실행
        :param sql:
        :return:
        """
        session = cls.session_maker()
        try:
            session.execute(sql)
            session.commit()
        except Exception as e:
            session.rollback()
            traceback.print_exc()
            raise e
        finally:
            session.close()

    def emit(self, record):
        self.format(record)
        if record.exc_info is not None:
            # trackback 객체는 별로도 str화
            record.exc_info = ''.join(traceback.format_exception(*record.exc_info))

        # 메세지 특수문자 처리 - ''2개 붙이면 1개 처리됨
        record.message = record.message.replace('\'', '\'\'')

        insert_sql = f"INSERT INTO {self.db_table} (" \
                     f"container, process, process_name, " \
                     f"thread, thread_name, " \
                     f"name, pathname, func_name, lineno, " \
                     f"levelno, levelname, time, " \
                     f"message, exc_info, exc_text, stack_info" \
                     f") VALUES (" \
                     f"'{self.container}', '{record.process}', '{record.process}', " \
                     f"'{record.thread}', '{record.threadName}', " \
                     f"'{record.name}', '{record.pathname}', '{record.funcName}', '{record.lineno}', " \
                     f"'{record.levelno}', '{record.levelname}', '{datetime.now()}', " \
                     f"'{record.message}', '{record.exc_info}', '{record.exc_text}', '{record.stack_info}'" \
                     f");"
        self.execute_sql(insert_sql)
